package multidaycalendarviewwrapper;

import main.java.com.github.sundeepk.multidaycalendarview.MultiDayCalendarView;
import main.java.com.github.sundeepk.multidaycalendarview.domain.Event;

import android.content.Context;
import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;

import android.view.View;
import android.graphics.Color;

import android.os.Bundle;
import android.app.Activity;


import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

//@ActivityObject
@Events(values={"event_selected(eventname As String, eventdetail As String)", "calendar_scrolling(firstdateshown As String)", 
                "event_added(eventstartdatetime As Long, clickeventname As String, clickeventdetail As String, clickeventcolor As Int)"})
@Author("Github: Sundeep Kahlon, Wrapped by: Johan Schoeman")
@Version(1.00f)
@ShortName("MultiDayCalendarView")
@DependsOn(values={"android-support-v4"})
public class multidayCalendarViewWrapper extends ViewWrapper<MultiDayCalendarView> implements DesignerCustomView {


	private BA ba;
	private String eventName;
    private MultiDayCalendarView cv;

	
	public void Initialize(BA ba, String EventName) {

		_initialize(ba, null, EventName);
	}

	@Hide
	public void _initialize(BA ba, Object activityClass, String EventName) {
		this.eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		cv = new MultiDayCalendarView(ba.context);
        this.setObject(cv);
		
		init();

 	}
	
	private void init() {
		long now = System.currentTimeMillis();
        cv.scrollTo(new Date(now));
        cv.setTimeZoneAndLocale(TimeZone.getTimeZone("GMT"), Locale.getDefault());
        cv.setCalendarListener(new MultiDayCalendarView.MultiDayCalendarListener() {		
            @Override
            public void onNewEventCreate(long eventStartDateTime) {
                //BA.Log("onNewEventCreate " + eventStartDateTime);
                cv.addEvent(eventStartDateTime, new Event<>(clickeventname, clickeventdetail, clickeventcolor));
                if (ba.subExists(eventName + "_event_added")) {
			  	  ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_event_added", true, new Object[]{eventStartDateTime, clickeventname, clickeventdetail, clickeventcolor});
                }					
            }

            @Override
            public void onEventSelect(long eventStartDateTime, Event selectedEvent) {
              if (ba.subExists(eventName + "_event_selected")) {
				ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_event_selected", true, new Object[]{selectedEvent.getEventName(),selectedEvent.getData().toString()});
              }					
             }

            @Override
            public void onCalendarScroll(Date firstDateTimeShownInCalendarView) {
              if (ba.subExists(eventName + "_calendar_scrolling")) {
				ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_calendar_scrolling", true, new Object[]{firstDateTimeShownInCalendarView.toString()});
              }		

            }
        });
	}
	
		
	//programmatically add view
    @Hide
	public void AddToParent(ViewGroup Parent, @Pixel int left, @Pixel int top, @Pixel int width, @Pixel int height) {
		Parent.addView(cv, new BALayout.LayoutParams(left, top, width, height));
	}
	
	//this method cannot be hidden.
  
        
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, anywheresoftware.b4a.objects.collections.Map props) {
		ViewGroup vg = (ViewGroup) base.getObject().getParent();
		AddToParent(vg, base.getLeft(), base.getTop(), base.getWidth(), base.getHeight());
		base.RemoveView();
	}

	public void setLeft(int left)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.left = left;
		cv.getParent().requestLayout();
	}

	public int getLeft()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.left;
	}
	public void setTop(int top)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.top = top;
		cv.getParent().requestLayout();
	}
 
	public int getTop()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.top;
	}
	
	public void setWidth(int Width)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.width = Width;
		cv.getParent().requestLayout();
	}

	public int getWidth()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.width;
	}
	
	public void setHeight(int height)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.height = height;
		cv.getParent().requestLayout();
	}

	public int getHeight()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.height;
	}
	

	/**
     * Add an Event to the calendar
	 * The date and time converted to seconds (use the date and hour only - make minutes and seconds zero)
	 * Eventname = the name of the Event
	 * detail = the detail of the event
	 * color = the color of the cell that houses the Event
     */			
	public void addEvent(long seconds, String Eventname, String detail, int color) {
		
		cv.addEvent(seconds, new Event<>(Eventname, detail, color));
		cv.invalidate();		
	}

	/**
     * The name of an Event that will be added to an empty cell
	 * If you touch and empty cell (will be color highlighted) and then touch the same cell again an event will be added
	 * The event that will be added is based on what have been set with:
	 * ClickEventName = clickeventname,
	 * ClickEventDetail = clickeventdetail, and
	 * ClickEventColor = clickeventcolor
     */				
	private String clickeventname = "";
	public void setClickEventName(String clickeventname) {
		this.clickeventname = clickeventname;
	}
	
	/**	
     * The detail of an Event that will be added to an empty cell
	 * If you touch and empty cell (will be color highlighted) and then touch the same cell again an event will be added
	 * The event that will be added is based on what have been set with:
	 * ClickEventName = clickeventname,
	 * ClickEventDetail = clickeventdetail, and
	 * ClickEventColor = clickeventcolor
     */				
	private String clickeventdetail = "";
	public void setClickEventDetail(String clickeventdetail) {
		this.clickeventdetail = clickeventdetail;
	}	
	
	
	/**	
     * The color of an Event that will be added to an empty cell
	 * If you touch and empty cell (will be color highlighted) and then touch the same cell again an event will be added
	 * The event that will be added is based on what have been set with:
	 * ClickEventName = clickeventname,
	 * ClickEventDetail = clickeventdetail, and
	 * ClickEventColor = clickeventcolor
     */				
	private int clickeventcolor = 0xff00ffff;
	public void setClickEventColor(int clickeventcolor) {
		this.clickeventcolor = clickeventcolor;
	}	

	/**
     * Remove an Event from the Calendar
	 * epochTimeInSecsToTheClosetHour should be in "seconds"
     */		
    public void removeEvent(long epochTimeInSecsToTheClosetHour){
        cv.removeEvent(epochTimeInSecsToTheClosetHour);
        cv.invalidate();
    }	
	
	/**
     * The color of an empty cell to chenge to when it is clicked on
	 * If you touch the same cell again an event will be added
	 * The event that will be added is based on what have been set with:
	 * ClickEventName = clickeventname,
	 * ClickEventDetail = clickeventdetail, and
	 * ClickEventColor = clickeventcolor
     */				
	public void setDefaultEventSelectColor(int defaultEventRectSelectColor) {
		cv.setDefaultEventRectSelectColor(defaultEventRectSelectColor);
		cv.invalidate();
	}	


	/**
     * Default is 15
     */			
	public void setDayOfWeekTextSize(int dayOfWeekTextSize){
		cv.setDayOfWeekTextSize(dayOfWeekTextSize);
		cv.invalidate();
	}
	
	/**
     * Default is 23
     */	
	public void setDayOfMonthTextSize(int dayOfMonthTextSize){
		cv.setDayOfMonthTextSize(dayOfMonthTextSize);
		cv.invalidate();
	}	
	
	/**
     * Default is 13
     */	
	public void setTimeTextSize(int timeTextSize){
		cv.setTimeTextSize(timeTextSize);
		cv.invalidate();
	}	

	
	/**
     * Default is 13
     */	
	public void setEventTextSize(int eventTextSize){
		cv.setEventTextSize(eventTextSize);
		cv.invalidate();
	}	
	
	/**
     * Default is 0xffff0000 (or red)
     */		
	public void setDateTextColor(int DateTextColor) {
		cv.setDateTextColor(DateTextColor);
		cv.invalidate();
	}	
	
	/**
     * Default is 0xff00ff00 (or green)
     */	
	public void setTimeTextColor(int TimeTextColor) {
		cv.setTimeTextColor(TimeTextColor);
		cv.invalidate();
	}			


   
}
